@@ -7,15 +7,14 @@ from __future__ import absolute_import
 from __future__ import print_function
 
 import abc
-import json
 from collections import OrderedDict
 
-import six
 # noinspection PyUnresolvedReferences
 from six.moves import range, zip
 
 from pybufrkit.constants import INDENT_CHARS, PARAMETER_TYPE_TEMPLATE_DATA
-from pybufrkit.errors import UnknownDescriptor
+from pybufrkit.errors import PyBufrKitError
+from pybufrkit.utils import fixed_width_repr_of_int
 from pybufrkit.bufr import BufrMessage
 from pybufrkit.descriptors import (Descriptor, ElementDescriptor, FixedReplicationDescriptor,
                                    DelayedReplicationDescriptor, OperatorDescriptor,
@@ -26,18 +25,6 @@ from pybufrkit.templatedata import (TemplateData, NoValueDataNode, SequenceNode,
 from pybufrkit.dataquery import QueryResult
 
 
-# Encode bytes as string for Python 3
-class EntityEncoder(json.JSONEncoder):
-    def default(self, o):
-        if isinstance(o, six.binary_type):
-            return o.decode(encoding='latin-1')
-
-        return json.JSONEncoder.default(self, o)
-
-
-JSON_DUMPS_KWARGS = {'encoding': 'latin-1'} if six.PY2 else {'cls': EntityEncoder}
-
-
 class Renderer(object):
     """
     This class is the abstract base Renderer. A renderer provides the contract
@@ -61,7 +48,7 @@ class Renderer(object):
         elif isinstance(obj, QueryResult):
             return self._render_query_result(obj)
         else:
-            raise RuntimeError('Unknown object {} for rendering'.format(type(obj)))
+            raise PyBufrKitError('Unknown object {} for rendering'.format(type(obj)))
 
     @abc.abstractmethod
     def _render_bufr_message(self, bufr_message):
@@ -119,12 +106,17 @@ class FlatTextRenderer(Renderer):
                     )
 
                 if idx in bitmap_links:
-                    ret.append('{:4d} {:65.65} -> {:<6d} {!r}'.format(
-                        idx + 1, self._render_descriptor(descriptor), bitmap_links[idx] + 1, value)
+                    ret.append('{} {:64.64} -> {} {!r}'.format(
+                        fixed_width_repr_of_int(idx + 1, 5),
+                        self._render_descriptor(descriptor),
+                        fixed_width_repr_of_int(bitmap_links[idx] + 1, 6, pad_left=False),
+                        value)
                     )
                 else:
-                    ret.append('{:4d} {:75.75} {!r}'.format(
-                        idx + 1, self._render_descriptor(descriptor), value)
+                    ret.append('{} {:74.74} {!r}'.format(
+                        fixed_width_repr_of_int(idx + 1, 5),
+                        self._render_descriptor(descriptor),
+                        value)
                     )
         return '\n'.join(ret)
 
@@ -175,7 +167,7 @@ class FlatTextRenderer(Renderer):
             lines.append('{}{} {} bits'.format(indent, descriptor, descriptor.nbits))
 
         else:
-            raise UnknownDescriptor('{}'.format(descriptor))
+            lines.append('{}{} (UNDEFINED)'.format(indent, descriptor))
 
         return lines
 
@@ -196,15 +188,15 @@ class FlatJsonRenderer(Renderer):
             section_data = []
             for parameter in section:
                 if parameter.type == PARAMETER_TYPE_TEMPLATE_DATA:
-                    section_data.append(json.loads(self._render_template_data(parameter.value)))
+                    section_data.append(self._render_template_data(parameter.value))
                 else:
                     section_data.append(parameter.value)
             data.append(section_data)
 
-        return json.dumps(data, **JSON_DUMPS_KWARGS)
+        return data
 
     def _render_template_data(self, template_data):
-        return json.dumps(template_data.decoded_values_all_subsets, **JSON_DUMPS_KWARGS)
+        return template_data.decoded_values_all_subsets
 
     def _render_descriptor(self, descriptor):
         raise NotImplementedError()
@@ -213,7 +205,7 @@ class FlatJsonRenderer(Renderer):
         ret = OrderedDict()
         for idx_subset in query_result.subset_indices():
             ret[idx_subset] = query_result.get_values(idx_subset, flat=True)
-        return json.dumps(ret, **JSON_DUMPS_KWARGS)
+        return ret
 
 
 class NestedJsonRenderer(Renderer):
@@ -228,13 +220,14 @@ class NestedJsonRenderer(Renderer):
             for parameter in section:
                 parameter_data = {'name': parameter.name}
                 if parameter.type == PARAMETER_TYPE_TEMPLATE_DATA:
-                    parameter_data['value'] = json.loads(self._render_template_data(parameter.value))
+                    parameter_data['value'] = self._render_template_data(parameter.value)
                 else:
                     parameter_data['value'] = parameter.value
                 section_data.append(parameter_data)
             data.append(section_data)
 
-        return json.dumps(data, **JSON_DUMPS_KWARGS)
+        # return data
+        return data
 
     def _render_template_data(self, template_data):
         ret = []
@@ -246,13 +239,13 @@ class NestedJsonRenderer(Renderer):
                     template_data.decoded_values_all_subsets[idx_subset],
                 )
             )
-        return json.dumps(ret, **JSON_DUMPS_KWARGS)
+        return ret
 
     def _render_query_result(self, query_result):
         ret = OrderedDict()
         for idx_subset in query_result.subset_indices():
             ret[idx_subset] = query_result.get_values(idx_subset)
-        return json.dumps(ret, **JSON_DUMPS_KWARGS)
+        return ret
 
     def _render_template_data_nodes(self, decoded_nodes, decoded_descriptors, decoded_values):
         ret = []
